﻿
namespace viasat_cdr
{
    class Program
    {
        static void Main(string[] args)
        {
            CDRproj.BerCodec codec = new CDRproj.BerCodec();
            CDRproj.CDR.CallEventRecord rec = new CDRproj.CDR.CallEventRecord();

            // Example where multiple CDR records are stored in file blocks of 1KB each. 
            // Each block starts with a four-byte header (vendor specific), 
            // followed by the CDR record itself (this is the ASN.1 part that we can decode)
            // followed by trailing bytes (vendor specific), until the next block.
            // 
            //   0  1  2  3  4        ...                            1024 
            //  |   header    |  CDR record   |    trailing bytes   |  next block
            //   xx xx xx xx XX XX XX ... XX 00 00 00 00 ... 00 00 00  

            const int blockLength = 1024;
            const int headerLength = 4;

            byte[] buffer = new byte[blockLength];

            int recLength = 0;

            // read in blocks of 1024 bytes from a file into a buffer, decode the buffer  
            using (System.IO.Stream fs = System.IO.File.OpenRead(@"..\..\..\cdr.ber"))
                while (fs.Read(buffer, 0, buffer.Length) > 0)
                {
                    recLength = codec.Decode(buffer, headerLength, rec); // skip over the header

                    System.Console.WriteLine("Record length {0}: {1}", recLength, rec);
                }

            // alternative technique is to decode directly from a file and skip trailing 00's    
            using (System.IO.Stream fs = System.IO.File.OpenRead(@"..\..\..\cdr.ber"))
                while (fs.Read(buffer, 0, headerLength) > 0) // skip over the header
                {
                    recLength = codec.Decode(fs, rec);

                    System.Console.WriteLine("Record length {0}: {1}", recLength, rec);

                    // skip trailing 00's
                    int b = 0;
                    while (b == 0)
                        b = fs.ReadByte();

                    if (b > 0) // non-zero byte - next block/record
                        fs.Seek(-1, System.IO.SeekOrigin.Current); // roll back one byte
                    else
                        break; //EOF
                }
        }
    }
}
